/*
 * alibavaService.cc
 *
 *  Created on: Jul 13, 2012
 *      Author: lacasta
 */
#include <string>
#include <sstream>
#include <deque>
#include <glib.h>

#include "alibavaService.h"

#include <gtk/gtk.h>
#include "../src/alibava_logo.h"


#include "alibava_wsdl.h"

#include "tabs.h"
#include "jscript_min.h"
#include "blue.h"
#include "favicon.h"

#include <DataFormat.h>
#include <ConfigFactory.h>
#include <ScanConfig.h>
#include <Analysis.h>
#include <DAQmanager.h>
#include "../gtkhst/histogram.h"
#include "../gtkhst/profile.h"

int set_parameter_value(const std::string &name, const std::string &value);
class SoapDataReceiver : public DataReceiver
{
    private:
        soap *ss;
        int run_type;
        HstHistogram *hst;
        HstProfile *prfl;
        HstHistogram *heff;
    public:
        SoapDataReceiver(soap *_ss_, int run_type, int nbin, double xmin, double xmax);
        ~SoapDataReceiver();
        void serialize(int &size, unsigned char **data);
        void receive(const AlibavaData *);
        void define_scan(int nevts, int npts, double from, double to);
        void end_of_scan_point(double value, unsigned int ntrg);
};

int send_buffer_full(struct soap *soap, const char *buff, int nchar, const char *type)
{
    //int nchar = strlen(buff);
    const char *end_ptr = buff + nchar;
    const char *text = buff;

    soap->http_content = type; // HTTP header with text/xml content
    soap_response(soap, SOAP_FILE);
    for (;;)
    {
        if (text>=end_ptr)
            break;

        size_t r = end_ptr - text;
        if ( r > sizeof(soap->tmpbuf) )
            r = sizeof(soap->tmpbuf);

        memcpy(soap->tmpbuf, text,r);
        text += r;

        if (soap_send_raw(soap, soap->tmpbuf, r))
            break; // can't send, but little we can do about that
    }
    soap_end_send(soap);
    return SOAP_OK;
}

int send_buffer(struct soap *soap, const char *buff, const char *type)
{
    int nchar = strlen(buff);
    return send_buffer_full(soap, buff, nchar-1, type);
}

int copy_file(struct soap *soap, const char *name, const char *type)
{
    FILE *fd;
    size_t r;
    fd = fopen(name, "rb"); /* open file to copy */
    if (!fd)
        return 404; /* return HTTP not found */
    soap->http_content = type;
    if (soap_response(soap, SOAP_FILE)) /* OK HTTP response header */
    {
        soap_end_send(soap);
        fclose(fd);
        return soap->error;
    }
    for (;;)
    {
        r = fread(soap->tmpbuf, 1, sizeof(soap->tmpbuf), fd);
        if (!r)
            break;
        if (soap_send_raw(soap, soap->tmpbuf, r))
        {
            soap_end_send(soap);
            fclose(fd);
            return soap->error;
        }
    }
    fclose(fd);
    return soap_end_send(soap);
}


#include "AlibavaMonitor.h"
extern AlibavaMonitor __monitor__;


int send_status(struct soap *soap)
{
    if (soap_response(soap, SOAP_HTML))
        return soap->error;

    GString *str = g_string_new("<!-- -->\n");
    g_string_append_printf(str,
            "<p>Runtype: <b>%s</b></p>"
            "<p>"
            "<table>"
            "<tr><td width=\"75%%\">N. of triggers</td><td>%d</td></tr>"
            "<tr><td>Elapsed Time</td><td>%s</td></tr>"
            "<tr><td>Rate</td><td>%s</td></tr>"
            "<tr><td>Efficiency</td><td>%s %%</td></tr>"
            "</table>"
            "</p>",
            __monitor__.getRunType().c_str(),
            __monitor__.getNtrigger(),
            __monitor__.getTimeElapsed().c_str(),
            __monitor__.getRate().c_str(),
            __monitor__.getEfficiencyAsString(1).c_str()
    );

    int rc = soap_send(soap, str->str);
    if (!rc)
        rc = soap_end_send(soap);
    else
        rc = soap->error;

    g_string_free(str, TRUE);
    return rc;
}

const char hst_signal[] = { "<img src='/image/hstSignal' align='center'>\0" };
const char hst_hitmap[] = {"<img src='/image/hstHitmap' align='center'>\0"};
const char hst_temp[] = {"<img src='/image/hstTemp' align='center'>\0"};
const char hst_pedestal[] = {"<table><tr><td><img src='image/hstPedestal'></td><td><img src='image/hstNoise'></td></tr></table>\0"};
const char hst_cmmd[] = {"<table><tr><td><img src='image/hstCmmd'></td><td><img src='image/hstCmmdNoise'></td></tr></table>\0"};
const char info_block[] = {"<iframe src='?status' frameborder='0'/>\0"};
int send_info(struct soap *soap)
{
    if (soap_response(soap, SOAP_HTML))
        return soap->error;

    GString *str = g_string_new("");

    g_string_append_printf(str,
            "<html>\n"
            "<head>\n"
            "<meta name='Generator' content='alibava-gui'>\n"
            "<script src=\"jquery.tools.min.js\"></script>\n"
            "<link rel=\"stylesheet\" href=\"tabs.css\">\n"
            "<title>Alibava DAQ Monitor</title>\n"
            "<script type=\"text/javascript\">\n"
            "   window.onload = setupRefresh;\n"
            "   function setupRefresh()\n"
            "    {\n"
            "        setInterval(\"refreshBlock();\",5000);\n"
            "    }\n"
            "    function refreshBlock()\n"
            "    {\n"
            "       document.getElementById('infoBlock').innerHTML = \"%s\";\n"
//            "       var selected = $( \"#tabs\" ).tabs( 'option', 'selected' );\n"
//            "       if (selected=='0')\n\tdocument.getElementById('tabSignal').innerHTML = \"%s\";\n"
//            "       else if (selected=='1')\n\tdocument.getElementById('tabPedestals').innerHTML = \"%s\";\n"
//            "       else if (selected=='2')\n\tdocument.getElementById('tabHitmap').innerHTML = \"%s\";\n"
//            "       else if (selected=='3')\n\tdocument.getElementById('tabTemp').innerHTML = \"%s\";\n"
//            "       else if (selected=='4')\n\tdocument.getElementById('tabCmmd').innerHTML = \"%s\";\n"
                           "       var ddd = document.getElementById('tabSignal');\n"
                           "       if (!ddd.webkitHidden)\n"
                           "          ddd.innerHTML = \"%s\";\n"
                           "       ddd = document.getElementById('tabPedestals');\n"
                           "       if (!ddd.webkitHidden)\n"
                           "          ddd.innerHTML = \"%s\";\n"
                           "       ddd = document.getElementById('tabHitmap');\n"
                           "       if (!ddd.webkitHidden)\n"
                           "          ddd.innerHTML = \"%s\";\n"
                           "       ddd = document.getElementById('tabTemp');\n"
                           "       if (!ddd.webkitHidden)\n"
                           "          ddd.innerHTML = \"%s\";\n"
                           "       ddd=document.getElementById('tabCmmd');\n "
                           "       if (!ddd.webkitHidden)\n"
                           "          ddd.innerHTML = \"%s\";\n"
            "    }\n"
            "</script>\n"
            "</head>\n"
            "<body>\n",
            info_block, hst_signal, hst_pedestal, hst_hitmap, hst_temp, hst_cmmd);


    //time_t clk = time(0);
    str = g_string_append(str,"<div  id=\"main\">\n");
    g_string_append_printf(str,"<p><a href=\"http://www.alibavasystems.com\" target=\"_blank\"><img src=\"/image/logo\" align=\"center\"></a></p>\n");
    //str = g_string_append(str,"<p><a href=\"http://www.alibavasystems.com\" target=\"_blank\">Alibava Systems</p>\n");
    str = g_string_append(str,"</div>\n");

    str = g_string_append(str,"<div class=\"box\" id=\"infoBlock\" width=\"100%\"></div>\n\n");

    g_string_append_printf(str,
                          "<div id=\"tabs\">\n" \
                          "<!-- the tabs -->\n" \
                          "<ul class=\"tabs\">\n" \
                          "<li><a href=\"#\">Signal</a></li>\n" \
                          "<li><a href=\"#\">Pedestals</a></li>\n" \
                          "<li><a href=\"#\">HitMap</a></li>\n" \
                          "<li><a href=\"#\">Temperature</a></li>\n"
                          "<li><a href=\"#\">Noise/Common Mode</a></li>\n"
                          "</ul>\n" \
                          "<!-- tab \"panes\" -->\n" \
                          "<div class=\"panes\">\n" \
                          "<div id=\"tabSignal\">%s</div>\n" \
                          "<div id=\"tabPedestals\">%s</div>\n" \
                          "<div id=\"tabHitmap\">%s</div>\n" \
                          "<div id='tabTemp'>%s</div>\n" \
                          "<div id='tabCmmd'>%s</div>\n" \
                          "</div>\n" \
                          "</div>\n",
                          hst_signal,
                          hst_pedestal,
                          hst_hitmap,
                          hst_temp,
                          hst_cmmd
                          );

    str = g_string_append(str,
                          "<script>\n"
                          "$(function() {\n "
                          "    $(\"ul.tabs\").tabs(\"div.panes > div\");\n"
                          "});\n"
                          "refreshBlock();\n"
                          "</script>\n");




    str = g_string_append(str, "\n</body>\n</html>");
    int rc = soap_send(soap, str->str);
    if (!rc)
        rc = soap_end_send(soap);
    else
        rc = soap->error;

    g_string_free(str, TRUE);
    return rc;
}

int get_hst_image(struct soap *soap, const char *s)
{
    int rc=0;
    GError *err=0;
    gchar *buffer=0;
    size_t size;
    std::string img(s);
    if (img == "logo")
    {
        GdkPixbuf *pixbuf = gdk_pixbuf_new_from_inline(-1, alibava_logo, FALSE, NULL);
        gdk_pixbuf_save_to_buffer(pixbuf, &buffer, &size, "png", &err, NULL);
        if (err)
        {
            g_printerr ("%s: %s\n", "save_logo: ", err->message);
            g_error_free(err);
            if ( buffer )
                g_free(buffer);

            buffer = 0;
        }
        if (buffer)
        {
            rc = soap_send_raw(soap, buffer, size);
            if (!rc)
                rc = soap_end_send(soap);
            else
                rc = soap->error;

            g_free(buffer);
        }
        g_object_unref(pixbuf);
    }
    else
    {
        __monitor__.getHistogram(img, size, &buffer);
        if (buffer)
        {
            rc = send_buffer_full(soap, buffer, size, "image/png");
            g_free(buffer);
        }
        rc = 0;
    }
    return rc;
}

int http_get(struct soap *soap)
{
    //printf("webserver path: %s\n", soap->path);

    if (soap->path[1] == '\0')
        return send_info(soap);

    if ( !strcmp(soap->path, "/tabs.css"))
        return send_buffer(soap, tabs_css, "text/css");

    if ( !strcmp(soap->path, "/jquery.tools.min.js"))
        return send_buffer(soap, (const char *)jscript_min_js, "text/javascript");

    if ( !strcmp(soap->path, "/blue.png"))
            return send_buffer_full(soap, (const char *) blue_png, blue_png_len, "image/png");

    if ( !strcmp(soap->path, "/favicon.ico"))
            return send_buffer_full(soap, (const char *) favicon_ico, favicon_ico_len, "image/ico");


    if ( strstr(soap->path, "image/"))
        return get_hst_image(soap, strrchr(soap->path, '/')+1);


    char *s = strchr(soap->path, '?');
    if (s)
    {
        if ( !strcmp(s, "?wsdl"))
            return send_buffer(soap, alibava_wsdl, "text/xml");


        if ( strstr(s, "status") )
            return send_status(soap);

    }


    return SOAP_GET_METHOD;

}


alibavaService::alibavaService()
{
    // TODO Auto-generated constructor stub
    this->fget = http_get;
    setup_names();
}

alibavaService::~alibavaService()
{
    // TODO Auto-generated destructor stub
}

alibavaService::alibavaService(const struct soap &_soap) :alibavaSOAPService(_soap)
{
        setup_names();
}

alibavaSOAPService *alibavaService::copy()
{       alibavaSOAPService *dup = SOAP_NEW_COPY(alibavaService(*(struct soap*)this));
        return dup;
}


void alibavaService::setup_names()
{
    hstnames[1] = "hstSignal";
    hstnames[2] = "hstPedestal";
    hstnames[3] = "hstNoise";
    hstnames[4] = "hstHitmap";
    hstnames[5] = "hstTemp";
    hstnames[6] = "hstTime";
    hstnames[7] = "hstEvent";
    hstnames[8] = "hstCmmdNoise";
    hstnames[9] = "hstCmmd";
}

/// Web service operation 'getStatus' (returns error code or SOAP_OK)
int alibavaSOAPService::getStatus(int getStatusRequest, struct ns1__getStatusResponse &_param_1)
{
    return SOAP_OK;
}

/// Web service operation 'getMonitorData' (returns error code or SOAP_OK)
int alibavaSOAPService::getMonitorData(int getMonitorDataRequest, struct ns1__getMonitorDataResponse &_param_2)
{
    return SOAP_OK;
}

/// Web service operation 'getEventData' (returns error code or SOAP_OK)
int alibavaSOAPService::getEventData(int getEventDataRequest, struct ns1__getEventDataResponse &_param_3)
{
    return SOAP_OK;
}

/// Web service operation 'getEventHeaders' (returns error code or SOAP_OK)
int alibavaSOAPService::getEventHeaders(int getEventHeadersRequest, struct ns1__getEventHeadersResponse &_param_4)
{
    return SOAP_OK;
}

int alibavaSOAPService::startRun(ns1__DAQParam *runType, struct ns1__startRunResponse &_param_5)
{
    return SOAP_OK;
}
int alibavaSOAPService::stopRun(int stopRunRequest, struct ns1__stopRunResponse &_param_6)
{
    return SOAP_OK;
}

int alibavaSOAPService::getRunData(int getRunDataParam, struct ns1__getRunDataResponse &_param_7)
{
    return SOAP_OK;
}


int alibavaSOAPService::setDataFile(std::string setDataFileRequest, struct ns1__setDataFileResponse &_param_7)
{
	return SOAP_OK;
}

int alibavaSOAPService::setParameter(ns1__ParValue *setParameterRequest, struct ns1__setParameterResponse &_param_8)
{
    return SOAP_OK;
}

int alibavaSOAPService::getMask(int getMaskRequest, struct ns1__getMaskResponse &_param_10)
{
    return SOAP_OK;
}

int alibavaSOAPService::getHistogram(ns1__ParValue *histogramName, struct ns1__getHistogramResponse &_param_11)
{
    return SOAP_OK;
}
int alibavaSOAPService::startRunAsync(ns1__DAQParam *daqParam)
{
	return SOAP_OK;
}
int alibavaSOAPService::Reset(int val)
{
	return SOAP_OK;
}


/// Web service operation 'getStatus' (returns error code or SOAP_OK)
int alibavaService::getStatus(int getStatusRequest, struct ns1__getStatusResponse &S)
{
    size_t sz;
    ns1__Status *status = soap_instantiate_ns1__Status(this,-1,"Status","Status", &sz);
    S.status = status;
    S.status->nexpected = __monitor__.getNexpected();
    S.status->ntrigger = __monitor__.getNtrigger();
    S.status->rate = __monitor__.getRateAsNumber();
    S.status->time = __monitor__.getTimeElapsed();
    S.status->value = __monitor__.getValue();
    S.status->state = __monitor__.isRunning() ? 1 : 0;

    switch ( __monitor__.getRunTypeVal() )
    {
        case 1:
            S.status->run_USCOREtype = ns1__RunType__Calibration;
            break;
        case 2:
            S.status->run_USCOREtype = ns1__RunType__LaserScan;
            break;
        case 3:
            S.status->run_USCOREtype = ns1__RunType__Laser;
            break;
        case 4:
            S.status->run_USCOREtype = ns1__RunType__RadioactiveSource;
            break;
        case 5:
            S.status->run_USCOREtype = ns1__RunType__Pedestal;
            break;
        default:
            S.status->run_USCOREtype = ns1__RunType__None;
            break;
    }
    return SOAP_OK;
}

/// Web service operation 'getMonitorData' (returns error code or SOAP_OK)
int alibavaService::getMonitorData(int getMonitorDataRequest, struct ns1__getMonitorDataResponse &_param_2)
{
    return SOAP_OK;
}
/// Web service operation 'getEventData' (returns error code or SOAP_OK)
int alibavaService::getEventData(int getEventDataRequest, struct ns1__getEventDataResponse &_param_3)
{
    std::cout << "getEventData" << std::endl;
    return SOAP_OK;
}

/// Web service operation 'getEventHeaders' (returns error code or SOAP_OK)
int alibavaService::getEventHeaders(int getEventHeadersRequest, struct ns1__getEventHeadersResponse &_param_4)
{
    std::cout << "getEventHeaders" << std::endl;
    return SOAP_OK;
}

int alibavaService::getMask(int getMaskRequest, struct ns1__getMaskResponse &_param_10)
{
    Analysis *analysis = dynamic_cast<Analysis *>(ConfigFactory::instance()->get("analysis"));
    _param_10.getMaskResponse = analysis->mask();
    return SOAP_OK;
}

void alibavaService::set_up_run(ns1__DAQParam *inp)
{
    if ( inp->runType == 4 )
    {
        ConfigFactory *cfg = ConfigFactory::instance();
        CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(cfg->get("calibration"));
        cal->scan_charge(inp->sampleSize);
        if (cal->scan_charge())
        {
            ScanConfig C = cal->charge_scan();
            C.nevts( inp->numEvents );
            C.set( inp->nbin, inp->xmin, inp->xmax );
            cal->charge_scan(C);
        }
        else
        {
            ScanConfig C = cal->delay_scan();
            C.nevts( inp->numEvents );
            C.set( inp->nbin, inp->xmin, inp->xmax );
            cal->delay_scan(C);
        }
    }
    else if ( inp->runType == 5 )
    {
        ConfigFactory *cfg = ConfigFactory::instance();
        CalibrationConfig *cal = dynamic_cast<CalibrationConfig *>(cfg->get("calibration"));
        ScanConfig C = cal->charge_scan();
        C.nevts( inp->numEvents );
        C.set( inp->nbin, inp->xmin, inp->xmax );
        cal->charge_scan(C);
    }
    else if ( inp->runType == 3 )
    {
        ConfigFactory *cfg = ConfigFactory::instance();
        LaserSyncConfig *cal = dynamic_cast<LaserSyncConfig *>(cfg->get("lasersync"));
        cal->nevts( inp->numEvents );
        cal->set( inp->nbin, inp->xmin, inp->xmax );
    }
}

int alibavaService::startRun(ns1__DAQParam *inp, struct ns1__startRunResponse &out)
{
    if (inp)
    {
        unsigned char *data=0;
        int size;
        set_up_run(inp);
        SoapDataReceiver *R = new SoapDataReceiver(this, inp->runType, inp->nbin, inp->xmin, inp->xmax);
        DAQmanager::daq_manager_start_run(inp->runType, inp->numEvents, inp->sampleSize, R);
        R->serialize(size, &data);
        out.startRunResponse.__ptr = data;
        out.startRunResponse.__size = size;
        delete R;
    }
        //return send_startRun_empty_response();
    return SOAP_OK;
}

struct RunData
{
        alibavaService *server;
        ns1__DAQParam *inp;
};
std::deque<SoapDataReceiver *> __runData__;
gpointer start_run(gpointer indata)
{
    RunData *D = (RunData *)indata;

    SoapDataReceiver *R = new SoapDataReceiver(D->server,
                                               D->inp->runType,
                                               D->inp->nbin, D->inp->xmin, D->inp->xmax);
    DAQmanager::daq_manager_start_run(D->inp->runType, D->inp->numEvents, D->inp->sampleSize, R);
    __runData__.push_back(R);
    delete D->inp;
    delete D;
    return NULL;
}

int alibavaService::getRunData(int getRunDataParam, struct ns1__getRunDataResponse &out)
{
    if (__runData__.empty())
    {
        out.getRunDataResponse.__size = 0;
        out.getRunDataResponse.__ptr = 0;
    }
    else
    {
        unsigned char *data=0;
        int size  = 0;

        SoapDataReceiver *R = __runData__.front();
        R->serialize(size, &data);
        out.getRunDataResponse.__ptr = data;
        out.getRunDataResponse.__size = size;
        __runData__.pop_front();
        delete R;
    }
    return SOAP_OK;
}

int alibavaService::startRunAsync(ns1__DAQParam *daqParam)
{
    ns1__DAQParam *dpar = new ns1__DAQParam;
    set_up_run(daqParam);
    dpar->runType = daqParam->runType;
    dpar->numEvents = daqParam->numEvents;
    dpar->sampleSize = daqParam->sampleSize;
    dpar->nbin = daqParam->nbin;
    dpar->xmin = daqParam->xmin;
    dpar->xmax = daqParam->xmax;

    RunData *D = g_new0(RunData, 1);
    D->server = this;
    D->inp = dpar;
#if GLIB_CHECK_VERSION(2,32,0)
    g_thread_new("soap_start_run", (GThreadFunc)start_run, D);
#else
    GError *err=0;
    g_thread_create((GThreadFunc)start_run, D, TRUE, &err);
    if (err)
    {
        std::cout << "Could not create the run thread: " << err->message << std::endl;
        g_error_free(err);
        send_startRunAsync_empty_response(500);
    }
    else
#endif
        send_startRunAsync_empty_response(202);

    return SOAP_OK;
}




int alibavaService::stopRun(int stopRunRequest, struct ns1__stopRunResponse &rc)
{
    rc.stopRunResponse = 0;
    DAQmanager::daq_manager_stop_run();
    //return send_stopRun_empty_response();
    return SOAP_OK;
}

int alibavaService::Reset(int resetParam)
{
    DAQmanager::daq_manager_reset_board();

    send_Reset_empty_response(202);
    return SOAP_OK;
}

int alibavaService::setDataFile(std::string setDataFileRequest, struct ns1__setDataFileResponse &_param_7)
{
    _param_7.setDataFileResponse = 0;
    DAQmanager::set_daq_manager_data_file( setDataFileRequest );
    return SOAP_OK;
}

SoapDataReceiver::SoapDataReceiver(soap *_ss, int runType, int nbin, double xmin, double xmax)
    : ss(_ss), run_type(runType), hst(0), prfl(0), heff(0)
{
    if (nbin <= 0)
        nbin = 1;

    if ( xmin==xmax )
        xmax = xmin + 1;

    hst = hst_histogram_new(nbin, xmin, xmax, 0);

    Analysis *analysis = dynamic_cast<Analysis *>(ConfigFactory::instance()->get("analysis"));
    int nchan = analysis->mask() == 3 ? 256 : 128;
    prfl = hst_profile_new(nchan, 0., nchan, 0);
}
SoapDataReceiver::~SoapDataReceiver()
 {
     if (hst)
         g_object_unref(hst);

     if (prfl)
         g_object_unref(prfl);
 }
void SoapDataReceiver::receive(const AlibavaData *b)
{
    // Now we fill the histograms
    Analysis *analysis = dynamic_cast<Analysis *>(ConfigFactory::instance()->get("analysis"));
    const EventBlock *blk = b->get_block();
    int ic, ichan, ichip;
    double val;
    int nchan = analysis->mask() == 3 ? 256 : 128;
    int ioff = analysis->mask()==2 ? 128 : 0;

    for (ic=0; ic<nchan; ic++)
    {
        ichip = (ic+ioff) / 128;
        ichan = ic %128;
        if (run_type)
            val = blk->chip[ichip].signal[ichan] - analysis->ped(ic+ioff);
        else
            val = blk->chip[ichip].signal[ichan];

        hst_profile_fill(prfl, ic, val, 1.0);
        hst_histogram_fill(hst, val, 1.0);
    }
}

void SoapDataReceiver::define_scan(int nevts, int npts, double from, double to)
{
    double step = (to-from)/double(npts);
    double half_step;
    if (step==0)
    {
        to = 1.1*from;
        step = (to-from)/double(npts);
    }
    half_step = step/2.0;

    if (heff)
        g_object_unref(heff);

    heff = hst_histogram_new(npts, from-half_step, to+half_step, 0);
}

void SoapDataReceiver::end_of_scan_point(double value, unsigned int ntrg)
{
    if (heff)
    {
        hst_histogram_fill(heff, value, ntrg);

    }
}
void SoapDataReceiver::serialize(int &size, unsigned char **data)
{
    int nhst= heff ? 4 : 3;
    int nbin;
    double xmin, xmax;
    double *buff, *rms=0;
    std::streampos i,j;
    std::ostringstream ostr;


    g_object_get(G_OBJECT(hst), "nx", &nbin, "xmin", &xmin, "xmax", &xmax, "ptr", &buff, NULL );
    ostr.write((char *)&nhst, sizeof(int));
    ostr.write((char *)&nbin, sizeof(int));
    ostr.write((char *)&xmin, sizeof(double));
    ostr.write((char *)&xmax, sizeof(double));
    ostr.write((char *)buff, nbin*sizeof(double));

    g_object_get(G_OBJECT(prfl), "nx", &nbin, "xmin", &xmin, "xmax", &xmax,
                                 "ptr", &buff, "rms", &rms,
                                 NULL );

    ostr.write((char *)&nbin, sizeof(int));
    ostr.write((char *)&xmin, sizeof(double));
    ostr.write((char *)&xmax, sizeof(double));
    ostr.write((char *)buff, nbin*sizeof(double));

    ostr.write((char *)&nbin, sizeof(int));
    ostr.write((char *)&xmin, sizeof(double));
    ostr.write((char *)&xmax, sizeof(double));
    ostr.write((char *)rms, nbin*sizeof(double));
    g_free(rms);


    if (heff)
    {
        g_object_get(G_OBJECT(heff), "nx", &nbin, "xmin", &xmin, "xmax", &xmax, "ptr", &buff,  NULL );
        ostr.write((char *)&nbin, sizeof(int));
        ostr.write((char *)&xmin, sizeof(double));
        ostr.write((char *)&xmax, sizeof(double));
        ostr.write((char *)buff, nbin*sizeof(double));
    }
    std::string sout = ostr.str();
    size = sout.size();
    unsigned char *out = (unsigned char *)soap_malloc(ss, size);
    sout.copy((char *)out, size);

    *data = out;
}


int alibavaService::setParameter(ns1__ParValue *setParameterRequest, struct ns1__setParameterResponse &_param_8)
{
    set_parameter_value(setParameterRequest->name, setParameterRequest->value);
    return SOAP_OK;
}

int alibavaService::getHistogram(ns1__ParValue *histogramName, struct ns1__getHistogramResponse &_param_11)
{
    gchar *buffer=0;
    gsize size;

    __monitor__.getHistogram(histogramName->name, histogramName->value, size, &buffer);
    if (buffer)
    {
        _param_11.getHistogramResponse.__size= size;
        _param_11.getHistogramResponse.__ptr = (unsigned char *)soap_malloc(this, size);
        memcpy(_param_11.getHistogramResponse.__ptr, buffer, size);
        g_free(buffer);
    }
    return SOAP_OK;
}

