#include <iostream>
#include <iomanip>
#include "Alibava.h"
#include "ConfigFactory.h"
#include "BeetleConfig.h"
#include "Analysis.h"

static const char *var_name[] =
{
    "Itp","Ipre","Isha","Ibuf","Vfp","Vfs",
    "Ipipe","Vd","Vdcl","Ivoltbuf","Isf","Icurrbuf","Latency",
    "ROCtrl","RclkDiv","CompCtrl",
    0
};
static const unsigned short var_value[] =
{
    0x0000, 0x014c, 0x020A, 0x030a, 0x0400, 0x0500,
    0x0a0d, 0x0b82, 0x0c69, 0x0d14, 0x0e1a, 0x0f66, 0x1080,
    0x111a, 0x1200, 0x1309,
    0x0700, 0x0800
};


BeetleConfig::BeetleConfig() : _mask(3)
{
    int i;
    set_group("beetle");
    _comparator.set_group("comparator");
    for (i=0; var_name[i]; i++)
    {
    	unsigned short ix = (var_value[i] & 0xff00 )>>8;
        varlist[var_name[i]] = var_value[i];
        parindx[ ix ] = var_name[i];
    }
}


void BeetleConfig::cpy(const BeetleConfig &b)
{
    VarList::const_iterator ip;
    varlist.clear();
    varlist = b.varlist;
    parindx = b.parindx;
    _mask = b._mask;
    _comparator = b._comparator;
    set_group(b.get_group() );
}

unsigned short BeetleConfig::operator()(const std::string &var)
{
    VarList::iterator ip = varlist.find(var);
    if ( ip != varlist.end() )
        return ip->second;
    else
        return 0xffff;
}

int BeetleConfig::set(unsigned short code, unsigned short value, Alibava *A)
{
    int rc = 0;
    VarList::iterator ip = varlist.find( parindx[code] );
    if (ip!=varlist.end())
    {
        unsigned short val = code<< 8 | (value & 0xff);
        ip->second = val;
        if (A)
            A->beetle_config(1, A->mask(), &val );
    }
    else
        rc = -1;

    return rc;
}
int BeetleConfig::set(const std::string &name, unsigned short value, Alibava *A)
{
    int rc = 0;
    VarList::iterator ip = varlist.find( name );
    if (ip!=varlist.end())
    {
        unsigned short code = ip->second & 0xff00;
        unsigned short val = code | (value & 0xff);
        ip->second = val;
        if (A)
            A->beetle_config(1, A->mask(), &val );
    }
    else
        rc = -1;
    return rc;
}

void BeetleConfig::load_data()
{

    if (exists())
    {
        int i;
        for (i=0; var_name[i]; i++)
        {
            varlist[var_name[i]] = (var_value[i] & 0xff00) | (0xff &get_int(var_name[i]));
        }
        _mask = get_int("mask", 3);
    }
    if ( _comparator.key_file() != key_file() )
        _comparator.set_file(key_file());
    else
        _comparator.load_data();

    set_comparator_settings();
}
void BeetleConfig::save_data()
{
    VarList::const_iterator ip;
    for (ip=begin(); ip!=end(); ++ip)
    {
        set_int(ip->first.c_str(), (0xff & ip->second));
    }
    set_int("mask", _mask);
    _comparator.save_data();
}

ComparatorConfig &BeetleConfig::get_comparator_settings()
{
    VarList::iterator ip = varlist.find("CompCtrl");
    unsigned short CompCtrl = ip->second & 0xff;

    _comparator.setEnable( (CompCtrl & 9)==0 );
    _comparator.setMode( (CompCtrl & 16) ? ComparatorConfig::pulseMode : ComparatorConfig::trackMode );

    return _comparator;
}



unsigned short BeetleConfig::set_comparator_settings(const ComparatorConfig &comparator)
{
    _comparator = comparator;
    return set_comparator_settings();
}

unsigned short BeetleConfig::set_comparator_settings()
{
    VarList::iterator ip = varlist.find("CompCtrl");
    unsigned short CompCtrl = 0x1300;
    unsigned short ctrl = 0;
    if ( !_comparator.isEnabled() )
    {
        ctrl = 9;
    }
    //if ( _comparator.getPolarity() )
    //    ctrl |= 2;

    if ( _comparator.getMode() == ComparatorConfig::pulseMode )
        ctrl |= 16;


    CompCtrl |= ctrl;
    ip->second = CompCtrl;
    return CompCtrl;
}

void  BeetleConfig::mask(short x)
{
    Analysis *analysis = dynamic_cast<Analysis *>(ConfigFactory::instance()->get("analysis"));
    analysis->set_mask(x);
    _mask = x;
}

unsigned short BeetleConfig::set_value(const std::string param, unsigned short val, Alibava *A)
{
	unsigned short ret=0xffff;
	VarList::iterator ip = varlist.find(param);
	if (ip!=this->end())
	{
		ret = (ip->second & 0xff00 ) | val;
		ip->second = ret;
		if (A)
		{
			A->beetle_config(1, A->mask(), &ret );
		}
	}
	return ret;
}

void BeetleConfig::setup_easy(Alibava *A)
{
	int indx = 0;
	unsigned short values[4];

	values[indx++] = set_value("Ipre", 75, 0);
	values[indx++] = set_value("Isha", 10, 0);
	values[indx++] = set_value("Vfp",  20, 0);
	values[indx++] = set_value("Vfs", 150, 0);
	if (A)
		A->beetle_config(indx, A->mask(), values);

}

int BeetleConfig::config_beetle(int nval, unsigned short *val, Alibava *A)
{
	int rc = 0;
	unsigned short indx;

	std::map<unsigned short, std::string>::iterator ip;
	for (int i=0; i<nval; i++)
	{
		indx = (val[i] &0xff00)>>8;
		set(indx, val[i]);
	}
	if (A)
		rc = A->beetle_config(nval, mask(), val);

	return rc;
}

int BeetleConfig::config_beetle_comparator(Alibava *A)
{
    int cmask=0;

    if (A->firmware()>1)
    {
        // Set comparator settings
        unsigned short xxx = set_comparator_settings();
        unsigned short CompCtrl = xxx & ~2; //ip->second;



        A->beetle_config(1, A->mask(), &CompCtrl);

        for (int ichip=0; ichip<2; ichip++)
        {
            cmask = 1<<ichip;
            if (cmask & A->mask() )
                A->beetle_config_mask(ichip, _comparator.getMask(ichip));
        }
        for (int ichip=0; ichip<2; ichip++)
        {
            cmask = 1<<ichip;
            if (cmask & A->mask() )
                A->beetle_config_trim(ichip, _comparator.getTrim(ichip));
        }
        for (int ichip=0; ichip<2; ichip++)
        {
            cmask = 1<<ichip;
            if (cmask & A->mask() )
            {
            	unsigned short CCC = CompCtrl & ~2;
            	if (_comparator.getPolarity(ichip))
            		CCC |= 2;

            	A->beetle_config_comparator(ichip,
                                           _comparator.getMainTh(ichip),
                                           _comparator.getDeltaTh(ichip),
                                           _comparator.getVrc(ichip),
                                           _comparator.getIcomp(ichip),
										   CCC & 0xff);
            }
        }
        A->beetle_config(0, A->mask(),0);
    }

    return 0;

}

std::ostream &operator<<(std::ostream &os, const BeetleConfig &B)
{
    BeetleConfig::VarList::const_iterator ip;
    os << "Beetle config: mask " << B.mask() << std::endl;
    for (ip=B.begin();ip!=B.end();++ip){
        unsigned short indx = (ip->second & 0xff00)>>8;
        unsigned short val = (ip->second&0xff);
        os << ip->first << " - " << std::hex << ip->second << std::dec << " [ " << indx << ", " << val << " ]"<< std::endl;
    }
    return os;
}
