/*
 * CalibTrimRun.h
 *
 *  Created on: Jun 28, 2014
 *      Author: lacasta
 */

#ifndef CALIBTRIMRUN_H_
#define CALIBTRIMRUN_H_

#include <iosfwd>
#include <vector>
#include <map>
#include <DataReceiver.h>
#include <ComparatorConfig.h>
#include <gtkhst/gtkhistogram2d.h>
#include <signal_emitter.h>

class Alibava;
class ComparatorConfigTrimGUI;
class LineFitter
{
    private:
        std::vector< std::pair<double, double> > data;
        double offset, slope;

        void copy(const LineFitter &F);
    public:
        LineFitter(double offs=0, double slp=0) : offset(offs), slope(slp) {}
        LineFitter(const LineFitter &F ): offset(0), slope(0)
        {
            copy(F);
        }
        LineFitter &operator=(const LineFitter &F)
        {
        	if (&F != this)
        		copy(F);

        	return *this;
        }
        void add(double x, double y)
        {
            data.push_back( std::pair<double, double>(x, y) );
        }
        int fit();
        double eval(double x) const { return offset + x*slope; }
        double find(double y) const { return slope!=0 ? (y-offset)/slope : 0.0 ; }
        double getSlope() const { return slope; }
        double getOffset() const  { return offset; }
        int npoits() const { return data.size(); }

};
std::ostream &operator<<(std::ostream &os, const LineFitter &F);

class CalibTrimRun: public DataReceiver
{
    private:
		typedef std::map< int, LineFitter > ChanFitter;
        typedef std::pair<double, unsigned int> PointVal;
        typedef std::vector< PointVal > HstVal;
        enum ScanType {find_limits, gain_scan, trim_scan};
        int _nevts;
        int _mask;
        int _thrs;
        int _icomp;
        int _polarity;
        GtkWidget *hGain, *hTrim;
        Alibava *board;
        bool     _stop_run;
        HstVal trg_count;
        bool do_show_spread;
        bool do_trim_scan;
        bool do_find_thrs;
        bool do_comp_gain;
        bool do_current;
        bool do_thrs_scan;
        bool do_save_data;
        bool do_fix_trim;
        int  fix_trim_value;
        std::string folder;

        ChanFitter trim_fits;
        ChanFitter gain_fits;
        Emitter *emitter;
        ComparatorConfig *comp;
        ComparatorConfigTrimGUI *gui;

        void find_channel_interval(int &channel0, int &channel1, int &chip0, int &chip1);

    public:
        CalibTrimRun(Alibava *B, ComparatorConfig *cnf, int target, int nevts, int mask, GtkWidget *gain, GtkWidget *trim);
        virtual ~CalibTrimRun();

        void set_fix_trim(bool doit, int val) { do_fix_trim=doit; fix_trim_value=val; }
        bool fix_trim() const { return do_fix_trim; }
        int  get_fix_trim_value() const { return fix_trim_value; }
        void set_gui(ComparatorConfigTrimGUI *G) { gui=G; }
        int nevts() const { return _nevts; }
        int mask() const { return _mask; }
        int polarity() const { return _polarity; }
        int icomp() const { return _icomp; }
        void polarity(int x) { _polarity = (x>=0) ? 1 : -1; }
        void mask( int v) { _mask=v; }
        void threshold(int val) { _thrs = val; }
        void nevts(int val) { _nevts = val; }
        void icomp(int val) { _icomp = val; }
        virtual void define_scan(int nevts, int npts, double from, double to);
        virtual void end_of_scan_point(double value, unsigned int ntrg);

        void find_scan_limits(int vfrom, int ichip, int step, int &istart, int &iend);
        void find_comparator_gain();
        void find_trim_dacs();
        void find_current_setting();
        void threshold_scan();
        void show_spread(bool fixed_trims=false);
        void start_calib();
        double compute_trim_corr(int ithmain, bool save);
        double find_threshold();
        void todo_list(bool show_spread, bool do_trim, bool find_thrs, bool comp_gain, bool current, bool scan);
        void stop();
        const ComparatorConfig &get_comparator_config() const { return *comp; }

        void save_data(bool z) { do_save_data=z; }
        bool save_data() const { return do_save_data; }
        void set_output_folder(const std::string &f) { folder = f; }
        const std::string get_output_folder() const { return folder; }

        Emitter* getEmitter()
        {
            return emitter;
        }
        Alibava *get_board() { return board; }
};

#endif /* CALIBTRIMRUN_H_ */
