/*
 * ComparatorConfig.cc
 *
 *  Created on: Jun 4, 2013
 *      Author: lacasta
 */

#include "ComparatorConfig.h"

ChipCompConfig::ChipCompConfig()
    : ConfigFile("ChipConfig"),
      _mainth(0), _deltath(5), _Vrc(0), _icomp(15), _polarity(0),  _mask(128)
{
    for (int i=0; i<128; i++)
    {
        _trim[i] = 0;
        _offset[i] = 0.0;
        _slope[i] = 0.0;
        _gain[i] = 0.0;
    }
}

ChipCompConfig::ChipCompConfig(const ChipCompConfig &C)
: ConfigFile("ChipConfig"),
  _mainth(0), _deltath(5), _Vrc(0), _icomp(15),_polarity(0), _mask(128)
{
    cpy(C);
}

ChipCompConfig &ChipCompConfig::operator=(const ChipCompConfig &C)
{
    if (this != &C)
        cpy(C);

    return *this;
}

void ChipCompConfig::cpy(const ChipCompConfig &C)
{
    _mainth = C._mainth;
    _deltath = C._deltath;
    _Vrc = C._Vrc;
    _icomp = C._icomp;
    _mask = C._mask;
    _polarity = C._polarity;

    for (int i=0; i<128; i++)
    {
        _trim[i] = C._trim[i];
        _slope[i] = C._slope[i];
        _offset[i] = C._offset[i];
        _gain[i] = C._gain[i];
    }
}

void ChipCompConfig::setTrimLine(int chan, double offset, double slope)
{
    int ichan = chan % 128;
    _offset[ichan] = offset;
    _slope[ichan] = slope;
}

void ChipCompConfig::getTrimLine(int chan, double &offset, double &slope)
{
    int ichan = chan % 128;
    offset = _offset[ichan];
    slope  = _slope[ichan];
}

void ChipCompConfig::load_data()
{
    if (exists())
    {
        setMainth( get_int("Ithmain", 0.0));
        setDeltath( get_int("Ithdelta", 0.0));
        setVrc( get_int("Vrs", 0.0));
        setIcomp( get_int("Icomp", 5.0));
        setPolarity( get_int("polarity"));

        int iv, length;
        int *v = get_int_list("mask", length);
        for (iv=0; iv<length; iv++)
        {
          if (v[iv])
              _mask.Set(iv);
          else
              _mask.Clr(iv);
        }

        delete [] v;

        v = get_int_list("trim", length);
        for (iv=0; iv<length; iv++)
            _trim[iv] = v[iv];

        delete [] v;

        double*w = get_double_list("trim_offset", length);
        for (iv=0; iv<length; iv++)
            _offset[iv] = w[iv];

        delete w;

        w = get_double_list("trim_slope", length);
        for (iv=0; iv<length; iv++)
            _slope[iv] = w[iv];

        delete w;

        w = get_double_list("gain", length);
        for (iv=0; iv<length; iv++)
        	_gain[iv] = w[iv];

        delete w;

    }
}

void ChipCompConfig::save_data()
{
    set_int("Ithmain", getMainth());
    set_int("Ithdelta", getDeltath());
    set_int("Vrs", getVrc());
    set_int("Icomp", getIcomp());
    set_int("polarity", getPolarity());

    int iv, v[128];
    for (iv=0; iv<128; iv++)
        v[iv] = _mask.Bit(iv) ? 1 : 0;

    set_int_list("mask", v, 128);

    for (iv=0; iv<128; iv++)
        v[iv] = _trim[iv];

    set_int_list("trim", v, 128);
    set_double_list("trim_offset", _offset, 128);
    set_double_list("trim_slope", _slope, 128);
    set_double_list("gain", _gain, 128);

}

void ChipCompConfig::setMask(const BitSet &b)
{
    if (b.nBits()==128)
        _mask = b;
}

ComparatorConfig::ComparatorConfig()
    : ConfigFile("comparator"), _enable(false),
      _mode(pulseMode), _target(6*1024)
{
    chip[0].set_group("Chip.0");
    chip[1].set_group("Chip.1");
}

ComparatorConfig::ComparatorConfig(const ComparatorConfig &C)
{
    cpy(C);
}

ComparatorConfig &ComparatorConfig::operator =(const ComparatorConfig &C)
{
    if (this!=&C)
        cpy(C);

    return *this;
}

void ComparatorConfig::cpy(const ComparatorConfig &C)
{
    ConfigFile::operator =(C);
    //*dynamic_cast<ConfigFile *>(this) = *dynamic_cast<const ConfigFile *>(&C);
    _enable = C._enable;
    _mode = C._mode;
    _target = C._target;
    for (int ichip=0; ichip<2; ichip++)
        chip[ichip] = C.chip[ichip];
}

ComparatorConfig::~ComparatorConfig()
{
    // TODO Auto-generated destructor stub
}

void ComparatorConfig::load_data()
{
    if (exists())
    {
        setEnable( get_bool("enable"));
        setMode( (CompMode)get_int("mode") );
        setTarget( get_int("target", 6*1024) );
    }
    for (int ichip=0; ichip<2; ichip++)
    {
        if (chip[ichip].key_file() != key_file())
            chip[ichip].set_file(key_file());
        else
            chip[ichip].load_data();
    }
}

void ComparatorConfig::save_data()
{
    set_bool("enable", isEnabled());
    set_int("mode", getMode());
    set_int("target", getTarget());
    for (int ichip=0; ichip<2; ichip++)
        chip[ichip].save_data();
}
