/*
 * DataFormat.cc
 *
 *  Created on: May 27, 2010
 *      Author: lacasta
 */
#include <cstring>
#include <iostream>
#include "DataFormat.h"

std::allocator<char> EventData::data_allocator;

AlibavaData::AlibavaData()
    : _value(0.0), _nchip(0), _firmware(0), _mask(0)
{
    memset((void *)&block, 0, sizeof(block));
}
AlibavaData::AlibavaData(const AlibavaData &D)
{
    _value = D._value;
    _nchip = D._nchip;
    _firmware = D._firmware;
    _mask  = D._mask;
    int sszz = sizeof(block);
    memcpy(&block, &D.block, sszz);
}

class OldFirmware : public EventData
{
    public:
        OldFirmware()
        {
            set_nchip(2);
        }
        virtual ~OldFirmware() {};
        int size_calib() { return 512; }
        int size_data() { return 518; }
        int read_data(USBport &);
};
int OldFirmware::read_data(USBport &port)
{
    int rc, out, i;
    int sz = size_calib()/2;

    for (i=0; i<16; i++)
        block.chip[0].header[i] = block.chip[1].header[i] = 0;

    rc = port.read( block.chip[0].signal, sz );
    if (rc < sz)
        return rc;

    out = port.read( block.chip[1].signal, sz );
    if ( out < 0)
        return out;

    return rc+out;
}


class NewFirmware : public EventData
{
    public:
        NewFirmware(int nchip)
        {
            set_nchip(nchip);
        }
        virtual ~NewFirmware() {}
        int size_calib() { return nchip()*288; }
        int size_data() { return 6 + nchip()*288; }
        int read_data(USBport &);
};


int NewFirmware::read_data(USBport &port)
{
    int rc;
    int sz = size_calib();
    rc = port.read( &block.chip[0], sz );
    return rc;
}



class Firmware_v3 : public NewFirmware
{
    public:
        Firmware_v3(int nchip) : NewFirmware(nchip) {}
        virtual ~Firmware_v3() {}
        int size_data() { return 10 + nchip()*288; }
};

AlibavaData::~AlibavaData()
{
}

void AlibavaData::set_nchip(int nc)
{
    _nchip = nc;
}

double AlibavaData::time() const
{
    unsigned short fpart =  tdc() & 0xffff;
    short ipart = (tdc() & 0xffff0000)>>16;
    if (ipart<0)
        fpart *= -1;
    double tt = 100.*(ipart + (fpart/65535.));
    return tt;
}


double AlibavaData::temperature() const
{
    double t = 0.12*temp() - 39.8;
    return t;
}

EventData::EventData()
{

}

EventData::~EventData()
{

}

EventData *EventData::factory(int nchip, int fw, int mask)
{
    EventData *out = 0;
    if (fw <= 0)
        out = new OldFirmware();
    else
    {
        if (fw<=2)
            out = new NewFirmware(nchip);
        else
            out = new Firmware_v3(nchip);
    }
    out->set_firmware(fw);
    out->set_mask(mask);
    return out;
}
