/*
 * USBserialwin32.cc
 *
 *  Created on: Jan 11, 2012
 *      Author: lacasta
 */
#include <glib.h>
#ifdef HAVE_CONFIG_H
#ifdef G_OS_WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#endif

#include <iostream>
#include <vector>
#include <unistd.h>
#include "Fifo.h"
#include "USBserialWnds.h"
#ifdef G_OS_WIN32
#include <windows.h>
#ifdef HAS_FTD2XX
#include <d2xx/ftd2xx.h>
#endif
#else
#include "d2xx/WinTypes.h"
#endif
#include "utils.h"

struct USBserial_priv
{
    int fd;
    HANDLE hfd;
    HANDLE hThreadTerm;
};

std::vector<USBserial_priv *> handle_list;

HANDLE get_handle_from_fd(int fd)
{
    return handle_list[fd]->hfd;
}
HANDLE get_thread_stopper_from_fd(int fd)
{
    return handle_list[fd]->hThreadTerm;
}

USBserialWnds::USBserialWnds()
    : fd(0), _priv(0)
{
    _priv = g_new0(struct USBserial_priv, 1);
}

USBserialWnds::USBserialWnds(const char *fnam)
    : USBport(fnam), fd(0), _priv(0)
{
    _priv = g_new0(struct USBserial_priv, 1);
    open(fnam);
}


USBserialWnds::~USBserialWnds() {
    g_free(_priv);
}

#ifdef G_OS_UNIX
int USBserialWnds::open(const std::string &fnam)
{
    return 0;
}
int USBserialWnds::close()
{
    return 0;
}
int USBserialWnds::read(void *buf, int size)
{
    return 0;
}
int USBserialWnds::write(void *buf, int size)
{
    return 0;
}
int USBserialWnds::flush()
{
    return 0;
}

int USBserialWnds::fildes() const
{
    return fd;
}
int USBserialWnds::setup_fifo(Fifo *)
{
    return 0;
}

void USBserialWnds::stop_thread()
{

}

unsigned int USBserialWnds::data_available(unsigned int size) { return 0; }
int USBserialWnds::wait_for_data(double tmout) { return 0; }

#else

#ifdef HAS_FTD2XX
LONG __connect__()
{
    FT_HANDLE fthandle;
    FT_STATUS ftstatus;
    DWORD numdevs=0;
    DWORD flags=0;
    DWORD locID=0;
    DWORD deviceID=0;
    DWORD type=0;
    DWORD ftdevice=0;
    LONG port_number=-1;
    char serialnumber[16];
    char description[64];
    DWORD deviceindex;
    unsigned int br = 115200, rd=5000, wr=5000;

    ftstatus = FT_CreateDeviceInfoList(&numdevs);
    if (ftstatus != FT_OK) {
        return -1;
    }
    if (numdevs > 0 ) {
        for (DWORD i = 0; i < numdevs; i++) {
            ftstatus = FT_GetDeviceInfoDetail(i,&flags,&type,&deviceID,&locID,
            		                          serialnumber,description,&fthandle);
            if (ftstatus == FT_OK) {
                if (deviceID == 0x04036001) {
                    deviceindex = i;
                    ftstatus = FT_Open(deviceindex, &fthandle);
                    ftstatus = FT_SetDataCharacteristics(fthandle,
                    		FT_BITS_8,
                    		FT_STOP_BITS_1,
                    		FT_PARITY_NONE);
                    ftstatus = FT_SetBaudRate(fthandle, br);
                    ftstatus = FT_SetTimeouts(fthandle, rd, wr);
                    ftstatus = FT_SetLatencyTimer(fthandle, 2);
                    ftstatus = FT_GetComPortNumber(fthandle, &port_number);
                    ftstatus = FT_Close(fthandle);

                    break;
                }
            }
            else {
                break;
            }
        }
        if (ftstatus == FT_OK) {
            return port_number;
        }
        else {
            return -1;
        }
    }
    else {
        return -1;
    }
}
#endif



int USBserialWnds::open(const std::string &fnam)
{
    char line[64];
    char comports[16][10]={"\\\\.\\COM1",  "\\\\.\\COM2",  "\\\\.\\COM3",  "\\\\.\\COM4",
                           "\\\\.\\COM5",  "\\\\.\\COM6",  "\\\\.\\COM7",  "\\\\.\\COM8",
                           "\\\\.\\COM9",  "\\\\.\\COM10", "\\\\.\\COM11", "\\\\.\\COM12",
                           "\\\\.\\COM13", "\\\\.\\COM14", "\\\\.\\COM15", "\\\\.\\COM16"};
    int i, j=-1;
    gchar *inam = g_strdup( fnam.c_str() );
    int port_num = -1;
#ifdef HAS_FTD2XX
    if ( ( port_num =  __connect__() ) < 0 )
    {
    	err_msg(_("Apparently there is no Alibava board connected"));
    	g_free(inam);
    	return -1;
    }
    if ( port_num > 0 )
    {
    	sprintf(line,"COM%d", port_num);
    	g_free(inam);
    	inam = g_strdup(line);
    }
#endif
    for (i=0; i<16;i++)
    {
        sprintf(line,"COM%d", i+1);
        if ( !strcmp(line, inam) )
        {
            j=i;
            break;
        }
    }
    if (j<0)
    {
        err_msg(_("Port %s does not exist"), inam);
        g_free(inam);
        return -1;
    }
    g_free(inam);

    _priv->hfd = CreateFile(comports[j],
            GENERIC_READ|GENERIC_WRITE,
            0,                          /* no share  */
            NULL,                       /* no security */
            OPEN_EXISTING,
#if WNDS_POLL
            FILE_FLAG_OVERLAPPED,        // we want overlapped operation
#else
            0,
#endif
            NULL);                      /* no templates */

    if(_priv->hfd == INVALID_HANDLE_VALUE )
    {
        err_msg(_("unable to open port %s"), line);
        return -1;
    }

    DCB port_settings;
    const char *baudr ="baud=9600 data=8 parity=N stop=1";

    memset(&port_settings, 0, sizeof(port_settings));  /* clear the new struct  */
    port_settings.DCBlength = sizeof(port_settings);

    if( !BuildCommDCBA(baudr, &port_settings))
    {
      err_msg(_("unable to set com port dcb settings"));
      CloseHandle(_priv->hfd);
      _priv->hfd = 0;
      return(1);
    }
    // Common settings
    port_settings.fOutxCtsFlow = FALSE;
    port_settings.fOutxDsrFlow = FALSE;
    port_settings.fDtrControl = DTR_CONTROL_DISABLE;
    port_settings.fDtrControl = FALSE;
    port_settings.fDsrSensitivity = FALSE;
    port_settings.fOutX = FALSE;
    port_settings.fInX = FALSE;
    port_settings.fNull = FALSE;
    port_settings.fRtsControl = RTS_CONTROL_DISABLE;
    port_settings.fAbortOnError = FALSE;

    if(!SetCommState(_priv->hfd, &port_settings))
    {
      err_msg(_("unable to set com port cfg settings"));
      CloseHandle(_priv->hfd);
      _priv->hfd = 0;
      return(1);
    }

    COMMTIMEOUTS Cptimeouts;

/**
    Cptimeouts.ReadIntervalTimeout         = MAXDWORD;
    Cptimeouts.ReadTotalTimeoutMultiplier  = 0;
    Cptimeouts.ReadTotalTimeoutConstant    = 0;
    Cptimeouts.WriteTotalTimeoutMultiplier = 0;
    Cptimeouts.WriteTotalTimeoutConstant   = 0;
*/
    Cptimeouts.ReadIntervalTimeout         = 20;
    Cptimeouts.ReadTotalTimeoutMultiplier  = 10;
    Cptimeouts.ReadTotalTimeoutConstant    = 100;
    Cptimeouts.WriteTotalTimeoutMultiplier = 10;
    Cptimeouts.WriteTotalTimeoutConstant   = 100;


    if(!SetCommTimeouts(_priv->hfd, &Cptimeouts))
    {
      err_msg(_("unable to set com port time-out settings"));
      CloseHandle(_priv->hfd);
      return(1);
    }

#ifdef WNDS_POLL
    SetCommMask(_priv->hfd, EV_RXCHAR|EV_TXEMPTY);
#endif

    fd = _priv->fd = handle_list.size();
    _priv->hThreadTerm = CreateEvent(0,0,0,0);
    handle_list.push_back(_priv);

    return 0;
}
int USBserialWnds::close()
{
    CloseHandle(_priv->hfd);
    CloseHandle(_priv->hThreadTerm);
    handle_list[_priv->fd] = 0;
    return 0;
}

int USBserialWnds::read(void *buf, int size)
{
	int n=0;
	BOOL rc = ReadFile(_priv->hfd, buf, size, (LPDWORD)((void *)&n), 0);
    if ( rc && n)
    	return n;

    char lastError[1024];
    FormatMessage(
            FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
            NULL,
            GetLastError(),
            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
            lastError,
            1024,
            NULL);
    err_msg("USBSerialWnds::read-> %s", lastError);
    return -1;
}
int USBserialWnds::write(void *buf, int size)
{
	BOOL iRet = false ;
	DWORD dwBytesWritten = 0;
	DWORD error;
	iRet = WriteFile (_priv->hfd, buf, size, &dwBytesWritten,0);
	if ( iRet )
		return dwBytesWritten;

	char lastError[1024];
	FormatMessage(
			FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			error,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			lastError,
			1024,
			NULL);
	err_msg("%s", lastError);
	return 0;
}

int USBserialWnds::flush()
{
    return 0;
}

/**
        * Polls for new data arriving.
        * Timeout: if <0 wait for ever
        *          if >0 wait for timeout milliseconds
        *
        *  Returns: 0  if there is data
        *           >0 if the call gave a timeout
        *           <0 an error
        */
#ifdef WNDS_POLL
int USBserialWnds::poll(int tmout)
{
	DWORD dwWait;
	DWORD dwEventMask=0;
	OVERLAPPED ov;
	memset(&ov,0,sizeof(ov));

	ov.hEvent = CreateEvent( 0,true,0,0);
	if (tmout<0)
		tmout = INFINITE;
	else
		tmout = 1000*tmout;


	while (true)
	{
		BOOL abRet = WaitCommEvent(_priv->hfd,&dwEventMask, &ov) ;
		if ( !abRet )
		{
			if( GetLastError () != ERROR_IO_PENDING)
			{
				char lastError[1024];
				FormatMessage(
						FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						GetLastError(),
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
						lastError,
						1024,
						NULL);
				err_msg("%s", lastError);
				return -1;
			}
		}

		dwWait = WaitForSingleObject (ov.hEvent, tmout);
		switch ( dwWait )
		{
			case WAIT_OBJECT_0:
			{
				DWORD dwMask;
				if (GetCommMask(_priv->hfd,&dwMask) )
				{
					if ( dwMask & EV_RXCHAR )
					{
						CloseHandle(ov.hEvent);
						return 0;
					}
					else if ( dwMask & EV_TXEMPTY)
					{
						ResetEvent ( ov.hEvent );
						continue;
					}
				}
			}
			break;
			case WAIT_TIMEOUT:
				return 1;
		}//switch
	}
	return 1;
}
#endif

int USBserialWnds::fildes() const
{
    return fd;
}
int USBserialWnds::setup_fifo(Fifo *F)
{
    return F->attach(this);
}

void USBserialWnds::stop_thread()
{
    SetEvent(_priv->hThreadTerm);
}

void USBserialWnds::wait_for_port_data(Fifo *F)
{
    bool abContinue = true;
    DWORD dwEventMask=0;
    char szTmp[1024];
    int iSize  = sizeof ( szTmp );

    OVERLAPPED ov;
    memset(&ov,0,sizeof(ov));
    ov.hEvent = CreateEvent( 0,true,0,0);
    HANDLE arHandles[2];
    arHandles[0] = _priv->hThreadTerm;

    DWORD dwWait;
    while (  !F->endpoll() && abContinue )
    {

        BOOL abRet = WaitCommEvent(_priv->hfd,&dwEventMask, &ov) ;
        if ( !abRet )
        {
            if( GetLastError () != ERROR_IO_PENDING)
            {
            	char lastError[1024];
            	    FormatMessage(
            	            FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
            	            NULL,
            	            GetLastError(),
            	            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
            	            lastError,
            	            1024,
            	            NULL);
            	    err_msg("%s", lastError);
            	    return;
            }
        }


        arHandles[1] = ov.hEvent ;

        dwWait = WaitForMultipleObjects (2,arHandles,FALSE,INFINITE);
        switch ( dwWait )
        {
            case WAIT_OBJECT_0:
                break;

            case WAIT_OBJECT_0 + 1:
            {
                DWORD dwMask;
                if (GetCommMask(_priv->hfd,&dwMask) )
                {
                    if ( dwMask == EV_TXEMPTY )
                    {
                        ResetEvent ( ov.hEvent );
                        continue;
                    }

                }

                //read data here...
                try
                {
                    BOOL abRet = false;

                    DWORD dwBytesRead = 0;
                    OVERLAPPED ovRead;
                    ovRead.hEvent = CreateEvent( 0,true,0,0);

                    do
                    {
                        ResetEvent( ovRead.hEvent  );
                        memset(szTmp,0,sizeof szTmp);
                        abRet = ReadFile(_priv->hfd,szTmp,sizeof(szTmp),&dwBytesRead,&ovRead);
                        if (!abRet )
                        {
                            abContinue = FALSE;
                            break;
                        }
                        if ( dwBytesRead > 0 )
                        {
                            F->write(szTmp, dwBytesRead, true);
                        }
                    } while ( dwBytesRead > 0 );
                    CloseHandle(ovRead.hEvent );
                }
                catch(...)
                {
                	char lastError[1024];
                	FormatMessage(
                			FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                			NULL,
                			GetLastError(),
                			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                			lastError,
                			1024,
                			NULL);
                	err_msg(_("Exception in thread\n%s"), lastError);
                	return;
                }

                ResetEvent ( ov.hEvent );
            }
            break;
        }//switch
    }
    return;
}

unsigned int USBserialWnds::data_available(unsigned int size)
{
	COMSTAT status;
	DWORD errors;
	ClearCommError(_priv->hfd, &errors, &status);

	return status.cbInQue;
}
int USBserialWnds::wait_for_data(double tmout)
{
#ifdef ExperiMental
	DWORD dwWait;
	DWORD dwEventMask=0;
	OVERLAPPED ov;
	memset(&ov,0,sizeof(ov));

	ov.hEvent = CreateEvent( 0,true,0,0);
	if (tmout<0)
		tmout = 1.e20;
	while (true)
	{
		BOOL abRet = WaitCommEvent(_priv->hfd,&dwEventMask, &ov) ;
		if ( !abRet )
		{
			if( GetLastError () != ERROR_IO_PENDING)
			{
				char lastError[1024];
				FormatMessage(
						FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						GetLastError(),
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
						lastError,
						1024,
						NULL);
				err_msg("%s", lastError);
				return 0;
			}
		}

		dwWait = WaitForMultipleObjects (1,&ov.hEvent,FALSE,int(tmout*1000));
		switch ( dwWait )
		{
			case WAIT_OBJECT_0:
			{
				DWORD dwMask;
				if (GetCommMask(_priv->hfd,&dwMask) )
				{
					if ( dwMask & EV_RXCHAR )
					{
						CloseHandle(ov.hEvent);
						return data_available(0);
					}
					else if ( dwMask & EV_TXEMPTY)
					{
						ResetEvent ( ov.hEvent );
						continue;
					}
				}
			}
			break;
		}//switch
	}
#endif
	int sz = data_available(0);
	return sz;
}

#endif
