#include <glib.h>
#ifdef HAVE_CONFIG_H
#ifdef G_OS_WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#endif

#include <iostream>
#include <string>
#include <sstream>
#include <cstdlib>
#include <cstring>
#include <csignal>
#include <getopt.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "ConfigFactory.h"
#include "AsciiDataFile.h"
#include "DAQmanager.h"
#include "AlibavaGUI.h"
#include "USBserial.h"
#include "USBemulator.h"
#include "USBFifo.h"
#include "USBserialWnds.h"
#include "Alibava.h"
#include "AlibavaMonitor.h"
#include "Progress.h"
#include "utils.h"
#include "path_utils.h"
#include "alibava_glade.h"
#include "alibava_icon.h"

#ifdef USE_SOAP
#include <AlibavaSOAP.h>
#endif

#ifdef G_OS_WIN32
void try_to_get_windows_font(void);
gchar *get_registry_key(const gchar *path, const gchar *key);
#endif

#ifdef __APPLE__
#include <gtkosxapplication.h>
extern GtkosxApplication *theMacApp;
#endif

#ifdef HAS_FTD2XX
#include "USBd2xx.h"
#endif
//#define USE_EMULATOR
//#define USE_FIFO


void test_emitter(GObject *obj, char *group, char *key, gpointer data)
{
    AlibavaGUI *A = (AlibavaGUI *)data;
    ConfigFactory *cfg = ConfigFactory::instance();

    if ( !strcmp(group, "DAQ"))
    {
        DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg->get("daq"));
        if ( !strcmp(key, "nevts") )
        {
            A->int2entry("entryMaxEvents", daq->nevts() );
        }
    }
    else if ( !strcmp(group, "laser"))
    {
        LaserConfig *laser = dynamic_cast<LaserConfig *>(cfg->get("laser"));
        if (!strcmp(key, "delay"))
        {
            A->int2entry("txtDelay", laser->delay() );
        }
    }

}


GMainLoop *__loop__=0;

void INTcatcher(int sig)
{
    printf("INTcatcher\n");
    if (sig == SIGINT)
    { // if single control-C, just set flag to stop run
        printf("...SIGINT\n");
        if (DAQmanager::get_the_manager())
        {
            printf("\nAaaarrrrrggggg !!!!!\n");
            DAQmanager::get_the_manager()->stop_run();
        }
        if (__loop__)
        {
           g_main_loop_quit(__loop__);
        }
    }
}


extern int __is_gui_session__;
// User data folder
const gchar *user_data_dir=0;

#if USE_INTLTOOL
#include <locale.h>
class LocaleSetter
{
    public:
        LocaleSetter()
        {
#ifdef G_OS_WIN32
            gchar *LOCALEDIR=get_registry_key("SOFTWARE\\AlibavaSL", "LOCALEDIR");
#endif
//#ifdef G_OS_UNIX
            setlocale (LC_ALL, "");
            bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
            bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
            textdomain (GETTEXT_PACKAGE);

            gchar *locale = getenv("LANG");
            if (locale)
                locale = g_strdup(locale);
            else
                locale = g_strdup("");

            //printf("locale is %s\n", setlocale(LC_ALL, locale));
            gtk_disable_setlocale();
            g_free(locale);
//#endif
#ifdef G_OS_WIN32
            g_free(LOCALEDIR);
#endif
        }
};
static LocaleSetter __set_locale__;
#endif

static char *glade_file=0;
const char *get_glade_file()
{
    return glade_file;
}

class MyReceiver : public DataReceiver
{
    private:
        int nevts;
        Progress prog;
    public:
        MyReceiver(int nn) :
            nevts(nn),
            prog(0, 1000, Progress::show_status | Progress::show_rate
                    | Progress::show_left | Progress::show_time | Progress::show_memory
                    | Progress::show_value)
        {
            prog.set_update(500000);
            prog.reset(1, nevts);
        }
        void receive(const AlibavaData *)
        {
            prog.increment();
        }
        void start() { prog.start(); }
        void stop() { prog.stop(); }

};


static void my_log (const gchar *log_domain,
                    GLogLevelFlags log_level,
                    const gchar *message,
                    gpointer user_data)
{
    if (!strcmp(log_domain,"libglade") && strstr(message,"unknown"))
        return;

    printf("MyLogger: %s\n", message);
    //    (*old_log)(log_domain,log_level,message,user_data);
}
static void setup_logger()
{
    g_log_set_handler ("GLib-GObject",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                        G_LOG_FLAG_FATAL |
                                        G_LOG_FLAG_RECURSION),
                       my_log, 0);
    g_log_set_handler ("Gtk",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                        G_LOG_FLAG_FATAL |
                                        G_LOG_FLAG_RECURSION),
                       my_log, 0);
    g_log_set_handler ("Gdk",
                          (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                           G_LOG_FLAG_FATAL |
                                           G_LOG_FLAG_RECURSION),
                          my_log, 0);
    g_log_set_handler ("GLib",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                        G_LOG_FLAG_FATAL |
                                        G_LOG_FLAG_RECURSION),
                       my_log, 0);
    g_log_set_handler ("libglade",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                               G_LOG_FLAG_FATAL |
                               G_LOG_FLAG_RECURSION),
                               my_log, 0);
    g_log_set_handler ("Pango",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                               G_LOG_FLAG_FATAL |
                               G_LOG_FLAG_RECURSION),
                               my_log, 0);

}

extern gchar *__daemon_path__;
gchar *__symbol_browser_path__ = 0;

int main(int argc, char **argv)
{
    /**
     *  Get the command line parameters
     */
    int igui=1;
    int imask=3;
    int iemulator=0;
    int idebug = 0;
    int isource = -1;
    int firmware = 3;
    int firmware_set = 0;
    int iplugin = 0;
    int idaemon = 0;
    int ififo=0;
    int id2xx=0;
    int iverbose=0;
    int nevts=100000;
    int sample_size=-1;
    int iport=10000;
    int ifileformat=(int)DataFile::AsciiFile;
    int ieasy=0;
    int option_index;
    bool no_dev_found=false;
    std::string cfg_file;
    std::string out_data_file;
    std::string device;

    // Make sure glib threads are initialized
    g_type_init();
    if (!g_thread_supported ())
        g_thread_init (NULL);

#ifdef HAVE_GIO
    GError *err=0;
    char *tmp;
    std::string cwd = PathUtils::getcwd();
    std::string prog_name( PathUtils::abspath(argv[0]) );
    std::string _prog_path, _daemon_path;
    _prog_path = PathUtils::dir_name( prog_name );

#ifdef G_OS_WIN32
    _daemon_path = PathUtils::find_bin( PathUtils::join(_prog_path, "alibavad.exe" ) );
#else
    _daemon_path = PathUtils::find_bin( PathUtils::join(_prog_path, "alibavad" ) );
    if (!PathUtils::exists(_daemon_path))
    {
        char *ddd = getenv("ALIBAVA_DAEMON");
        if (ddd)
        {
            std::string tmp = PathUtils::find_bin( ddd );
            if ( PathUtils::exists(tmp) )
                _daemon_path = tmp;
        }
        else
        {
            std::vector<std::string> ofiles;
            PathUtils::find_file("alibavad", ofiles, _prog_path,false);
            if (!ofiles.empty())
                _daemon_path = ofiles[0];
        }
    }

#endif
    __daemon_path__ = g_strdup(_daemon_path.c_str());
#ifdef G_OS_WIN32
    std::string _symbol_browser = PathUtils::find_bin( PathUtils::join(_prog_path, "plugin_dump.exe"));
    __symbol_browser_path__ = g_strdup(_symbol_browser.c_str());
#endif

    g_set_application_name("AlibavaGUI");
    user_data_dir = g_build_filename( g_get_user_special_dir( G_USER_DIRECTORY_DOCUMENTS ),
                                      g_get_application_name(),
                                      NULL);
    GFile *outdir = g_file_new_for_path(user_data_dir);
    gboolean dir_made = g_file_make_directory_with_parents(outdir, NULL, &err);
    if( err && err->code != G_IO_ERROR_EXISTS )
    {
        err_msg("Cannot create data folder: %s", err->message);
        g_error_free(err);
        err = 0;
    }

#endif


    static struct option long_options[] =
         {
#ifdef HAVE_HDF5
          {"hdf5", 0, &ifileformat, (int)DataFile::HDF5},
#endif
          {"ascii", 0, &ifileformat, (int)DataFile::AsciiFile},
          { "gui", 0, &igui, 1 },
          { "debug", 0, &idebug, 1},
          { "no-gui", 0,&igui, 0 },
          { "emulator", 0, &iemulator, 1},
          { "no-emulator", 0, &iemulator,0},
          { "calib", 0,&isource, RunManager::Calibration },
          { "laser", 0,&isource, RunManager::Laser },
          { "source", 0,&isource, RunManager::RadSource },
          { "pedestal", 0,&isource, RunManager::Pedestal },
          { "activate-plugin", 0, &iplugin, 1},
          { "fifo", 0, &ififo, 1},
          { "d2xx",0,&id2xx,1},
          { "no-fifo", 0, &ififo, 0},
          { "verbose", 0, &iverbose, 1},
          { "nevts", 1, 0, 'n' },
          { "sample", 1, 0, 's'},
          { "dev", 1, 0, 'd'},
          { "chip-mask", 1, 0, 'm'},
          { "easy", 0, &ieasy, 1},
          { "firmware",1,0,'f'},
          { "out", 1, 0, 'o'},
          { "soap-port", 1, 0, 'p'},
          { "daemon", 0, &idaemon, 1},
          { "help", 0, 0, 'h'},
          { 0, 0, 0, 0 } };

    while ( 1)
    {
        char c;
        option_index = 0;
        c = getopt_long(argc, argv, ":x:o:s:r:w:h:t:ud", long_options, &option_index);
        if (c == -1)
            break;
        switch (c)
        {
            case 0:
                break;
            case 'n':
                nevts = atoi(optarg);
                break;
            case 's':
                sample_size = atoi(optarg);
                break;
            case 'f':
            	firmware_set = 1;
                firmware = atoi(optarg);
                break;
            case 'd':
                device = optarg;
                break;
            case 'm':
                imask = atoi(optarg);
                if (imask<1 && imask>3)
                {
                    std::cout << _("Invalid chip mask: ") << imask << ". Setting it to 3" <<  std::endl;
                    imask = 3;
                }
                break;
            case 'o':
                out_data_file = PathUtils::abspath( optarg );
                break;
            case 'p':
                iport = atoi( optarg);
                break;
            case 'h':
                std::cout << "alibava-gui: XXXX" << std::endl;
                return 0;
            default:
                std::cout << "Error " << argv[optind-1] << " ";
                if (optind<argc)
                    std::cout << argv[optind];
                std::cout << " [Ughhhh !!!!]" << std::endl;
                return 1;
        }
    }
    while (optind<argc)
    {
        cfg_file = argv[optind++];
        if (!PathUtils::isabs(cfg_file))
        {
            if ( !PathUtils::exists(cfg_file) )
            {
                cfg_file = PathUtils::join(user_data_dir, cfg_file);
            }
            else
                cfg_file = PathUtils::abspath( cfg_file );

        }
    }

    if (ieasy)
    {
        idaemon = 0;
        igui = 1;
        imask = 1;
#ifdef HAVE_HDF5
        ifileformat = (int)DataFile::HDF5;
#endif
    }

    if (iverbose)
        __verbose__ = 1;

    __is_gui_session__ = igui;
    if (user_data_dir)
        chdir(   user_data_dir );


#ifdef G_OS_UNIX
    if ( device.empty() && !iemulator)
    {
        device = Alibava::find_dev_file();
        if (device.empty())
        {
            std::cout << "Starting in emulation mode" << std::endl;
            iemulator = 1;
            no_dev_found = true;
        }
    }
#else
    device = "XXXX";
#endif

    /*
     * Change the default for the emulator
     */
#ifdef G_OS_WIN32
    /// Emulator + Fifo do not work in windows
    if ( iemulator && (ififo == 999) )
        ififo = 0;

#endif

    setup_logger();
    /**
     * Decide which driver to use
     */
    USBport *port=0;
    USBport *usb=0;


    if (ififo)
    {
        if (iemulator)
        {
            usb = new USBemulator(firmware);
        }
        else
        {

#ifdef G_OS_UNIX
            usb = new USBserial;
#elif defined(G_OS_WIN32)
#ifdef HAS_FTD2XX
            if (id2xx)
                usb = new USBd2xx;
            else
#endif
            usb = new USBserialWnds;
#endif
        }
        if (id2xx)
        	port = usb;
        else
        	port = new USBFifo(*usb);
    }
    else
    {
        if (iemulator)
            port = new USBemulator(firmware);
        else
        {
#ifdef G_OS_WIN32
#ifdef HAS_FTD2XX
            if (id2xx)
                port = new USBd2xx;
            else
#endif
            	port = new USBserialWnds;
#else
                port = new USBserial;
#endif
        }
    }


    Alibava board(*port);
    PluginConfig *plg = (PluginConfig *)ConfigFactory::instance()->get("plugin");
    plg->active( (iplugin != 0) );
    if ( firmware_set )
    	board.force_firmware(firmware);

    BeetleConfig *beetle = dynamic_cast<BeetleConfig *>(ConfigFactory::instance()->get("beetle"));

#ifdef USE_SOAP
    // Start the SOAP server
    AlibavaSOAP *the_soap_server = new AlibavaSOAP(iport);
    the_soap_server->serve(true);
    usleep(100);
    if ( !the_soap_server->is_running() )
    {
        std::cout << "**** Could not start the SOAP server ******" << std::endl;
        if (the_soap_server->error() )
            std::cout << '\t' << the_soap_server->strerror() << std::endl;
    }
#endif
    if ( !out_data_file.empty() && out_data_file[0]!='/' && out_data_file[0]!='~' )
    {
        out_data_file = PathUtils::abspath( PathUtils::join(cwd, out_data_file) );
    }
    board.set_data_receiver(&__monitor__);
    if (idaemon)
    {
        DAQmanager D(1000, &board, RunManager::Pedestal);
        DAQmanager::set_the_manager( &D );
        D.file_format((DataFile::FileType)ifileformat);
        if (!cfg_file.empty())
            D.load_config(cfg_file.c_str(), false);

        beetle->mask(imask);
        D.open(device.c_str());


        signal(SIGINT, INTcatcher);
        __loop__ = g_main_loop_new(NULL, FALSE);
        g_main_loop_run(__loop__);
    }
    else if (!igui)
    {
        ConfigFactory *cfg = ConfigFactory::instance();
        DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg->get("daq"));
        if (isource<0)
            isource = RunManager::Pedestal;

        MyReceiver R(nevts);
        DAQmanager D(nevts, &board, (RunManager::RunType)isource);
        board.set_data_receiver(&R);
        DAQmanager::set_the_manager( &D );

        D.file_format((DataFile::FileType)ifileformat);
        D.set_data_file( out_data_file );
        if (!cfg_file.empty())
            D.load_config(cfg_file.c_str(), false);

        beetle->mask(imask);
        D.open(device.c_str());


        // overwrite what the config file says
        daq->nevts(nevts);
        daq->firmware( firmware );

        // no gui
        if ( D.prepare_for_run() )
            return -1;

        R.start();
        signal(SIGINT, INTcatcher);

        if ( D.acquire() )
            return -1;

        R.stop();
        D.stop_run();
    }
    else
    {
        // First thing to do is to initialize Gtk and Glade
        gtk_init(&argc, &argv);
        glade_init();

#ifdef MAC_INTEGRATION
        theMacApp = (GtkosxApplication *)g_object_new(GTKOSX_TYPE_APPLICATION, NULL);
#endif


        if (no_dev_found)
        {
            GtkDialog *dlg = GTK_DIALOG(
                    gtk_message_dialog_new_with_markup(0,
                                                       (GtkDialogFlags)(GTK_DIALOG_DESTROY_WITH_PARENT),
                                                       GTK_MESSAGE_INFO,
                                                       GTK_BUTTONS_OK,
                                                       _("<b>alibava-gui</b>:\nNo devices found.\nStarting in emulation mode"),
                                                       0));
            gtk_dialog_run(dlg);
            gtk_widget_hide(GTK_WIDGET(dlg));
            gtk_widget_destroy(GTK_WIDGET(dlg));
        }


#ifdef G_OS_WIN32
        try_to_get_windows_font();
#endif
//        char *app_font = 0;
//        GtkSettings *settings = gtk_settings_get_default();
//        g_object_get(G_OBJECT(settings), "gtk-font-name", &app_font, NULL);
//        std::cout << "::> Aplication font " << app_font << std::endl;
//        g_free(app_font);


        AlibavaGUI window(nevts, &board);
        window.easy( ieasy!= 0 );
        window.file_format((DataFile::FileType)ifileformat);
        window.set_debug( idebug!=0 );
        DAQmanager::set_the_manager( &window );

        ConfigFactory *cfg = ConfigFactory::instance();
        cfg->get("daq")->connect( G_CALLBACK(test_emitter), &window);
        cfg->get("laser")->connect(G_CALLBACK(test_emitter), &window);

        if (sample_size>0)
            window.sample_size(sample_size);

        window.init(alibava_glade, sizeof(alibava_glade) , "MainWindow");
        const guint8 *icon = (const guint8 *)alibava_icon_transp;
        if (ieasy) {
            icon = (const guint8 *)alibava_easy;
            gtk_window_set_title( GTK_WINDOW(window.get_main_window() ), "Educational Alibava System - EASY");
        }
        GdkPixbuf *pxIcon = gdk_pixbuf_new_from_inline(-1, icon, FALSE, NULL);
        g_object_set(G_OBJECT(window.get_main_window()),"icon", pxIcon,NULL);
        gtk_window_set_icon_name( GTK_WINDOW(window.get_main_window()), "alibava" );
#ifdef MAC_INTEGRATION
        pxIcon = gdk_pixbuf_new_from_inline(-1, icon, FALSE, NULL);
        gtkosx_application_set_dock_icon_pixbuf(theMacApp, pxIcon);
#endif

        if (!cfg_file.empty())
            window.load_config(cfg_file.c_str(), false);

        beetle->mask(imask);
        if (window.open(device.c_str()) )
        {
            err_msg(_("Could not open the device."));
        }

        if (ieasy)
        {
        	BeetleConfig *beetle = dynamic_cast<BeetleConfig *>(ConfigFactory::instance()->get("beetle"));
        	beetle->setup_easy(&board);
        }
#ifdef G_OS_WIN32
        /*
         * Apparently the windows alibavad dannot handle very big data samples.
         */
        if (iemulator)
        {
        	 DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg->get("daq"));
        	 daq->sample_size(10);
        }
#endif
        // Now we start the Gtk loop
        gtk_main();
    }
    delete port;

    std::cout.flush();
    if (usb)
        delete usb;

    return 0;
}



