#include <iostream>
#include <iterator>
#include <ctime>
#include <cerrno>
#include <cstdarg>
#include <cstring>
#include <gtk/gtk.h>
#include "GTimer.h"
#include "utils.h"
#ifdef G_OS_WIN32
#include <sys/types.h>
#include <sys/timeb.h>
#endif

unsigned int __verbose__ = 0;
std::map<void *, std::vector<std::string> > Verbose::_backtrace;


std::vector< std::string > &Verbose::get_bt(void *p)
{
    std::map<void *, std::vector<std::string> >::iterator ip = _backtrace.find(p);
    if (ip == _backtrace.end() )
    {
        std::vector<std::string> v;
        _backtrace[p] = v;
        ip = _backtrace.find(p);
    }
    return ip->second;
}

Verbose::Verbose(const std::string &msg, void *p, const char *func, int line)
   : _msg(msg), _func(func), _line(line), ptr(p), bct(get_bt(p))
{
    push_call(_func, line);
    if (__verbose__)
            p_trace_msg(_func.c_str(), _line, true,_msg.c_str());
}
Verbose::~Verbose()
{
    pop_call();
    if (__verbose__)
        p_trace_msg(_func.c_str(), _line, false,_msg.c_str());
}

void Verbose::push_call(const std::string &func, int line)
{
    static char buf[512];
    sprintf(buf, "%s:%d", func.c_str(), line);
    bct.push_back(buf);
}

void Verbose::pop_call()
{
    bct.pop_back();
}



void Verbose::backtrace(void *p)
{
    std::vector< std::string > &bct = get_bt(p);
    std::vector<std::string>::iterator ip;
    std::cout << "-------------------------------" << std::endl;
    std::cout << "BackTrace: " << std::endl;
    for (ip=bct.begin(); ip!=bct.end(); ip++)
        std::cout << *ip << std::endl;
    std::cout << "-------------------------------" << std::endl;
}

// WE can afford a few messages un allocated.
// It is the prices to pay to get the initialization
// messages in the gui
int __is_gui_session__=1;

gchar *check_for_end_of_line(gchar *msg)
{
    gchar *p = strrchr(msg,'\n');
    if ( p && *p == '\n' )
        return msg;
    else
    {
        int pos = strlen(msg)+1;
        gchar *out = (gchar *)g_realloc( (gpointer)msg, pos+2 );
        out[pos-1] = '\n';
        out[pos] = 0;
        return out;
    }
}

#if defined(ASCII)
void p_debug(const gchar *fname, const gchar *fmt, ...)
{
    gchar *msg;
    gchar *now;
    gchar *format;

    va_list args;

    va_start (args, fmt);

    now = get_time();
    format = g_strdup_printf("%s | Debug [%-15s] %s", now, fname, fmt);

    msg = g_strdup_vprintf(format, args);
    msg = check_for_end_of_line(msg);
    g_debug( msg );

    va_end (args);
    g_free(format);
    g_free( msg );
    g_free( now );
}

void p_err_msg(const gchar *func_name, int func_line, const gchar *fmt,...)
{
    gchar *format;
    gchar *msg;
    gchar *now;
    va_list args;

    va_start (args, fmt);
    now = get_time();
    if (errno)
        format = g_strdup_printf("%s | Error [%s:%d %s] %s", now, func_name, func_line, strerror(errno), fmt);
    else
        format = g_strdup_printf("%s | Error [%s:%d] %s", now, func_name, func_line, fmt);

    msg = g_strdup_vprintf(format, args);
    msg = check_for_end_of_line(msg);
    g_critical(msg);

    va_end (args);
    g_free(format);
    g_free( msg );
}

void p_info_msg(const gchar *func_name, int func_line, const gchar *fmt,...)
{
    gchar *format;
    gchar *msg;
    gchar *now;
    va_list args;

    va_start (args, fmt);
    now = get_time();
    format = g_strdup_printf("%s | Info [%s:%d] %s", now, func_name, func_line, fmt);

    msg = g_strdup_vprintf(format, args);
    msg = check_for_end_of_line(msg);
    g_message(msg);

    va_end (args);
    g_free(format);
    g_free( msg);
}


void write_msg(const gchar *fmt, ...)
{
    gchar *now;
    gchar *format;
    gchar *msg;
    va_list args;
    va_start (args, fmt);

    now = get_time();
    format = g_strdup_printf("%s | %s", now, fmt);
    msg = g_strdup_vprintf(format, args);

    msg = check_for_end_of_line(msg);
    g_print(msg);

    va_end (args);
    g_free(now);
    g_free(format);
}

#else


void x_debug(const gchar *fname, const gchar *fmt, ...)
{
    gchar *msg;
    gchar *now;
    gchar *format;

    va_list args;

    va_start (args, fmt);

    now = get_time();
    format = g_strdup_printf("%s | Debug [%-15s] %s", now, fname, fmt);

    msg = g_strdup_vprintf(format, args);
    msg = check_for_end_of_line(msg);
    __print_msg( msg, 0 );

    va_end (args);
    g_free(format);
    g_free( msg );
    g_free( now );
}

void x_err_msg(const gchar *func_name, int func_line, const gchar *fmt,...)
{
    gchar *format;
    gchar *msg;
    gchar *now;
    va_list args;

    va_start (args, fmt);
    now = get_time();
    if (errno)
        format = g_strdup_printf("%s | Error [%s:%d %s] %s", now, func_name, func_line, strerror(errno), fmt);
    else
        format = g_strdup_printf("%s | Error [%s:%d] %s", now, func_name, func_line, fmt);

    msg = g_strdup_vprintf(format, args);
    msg = check_for_end_of_line(msg);

    __print_msg( msg, 1 );

    va_end (args);
    g_free(msg);
    g_free(format);
}

void x_info_msg(const gchar *func_name, int func_line, const gchar *fmt,...)
{
    gchar *msg;
    gchar *format;
    gchar *now;
    va_list args;

    va_start (args, fmt);
    now = get_time();
    format = g_strdup_printf("%s | Info [%s:%d] %s", now, func_name, func_line, fmt);

    msg = g_strdup_vprintf(format, args);
    msg = check_for_end_of_line(msg);
    __print_msg( msg, 2 );

    va_end (args);
    g_free(msg);
    g_free(format);
}

void write_msg(const gchar *fmt, ...)
{
    gchar *now;
    gchar *format;
    gchar *msg;
    va_list args;
    va_start (args, fmt);

    now = get_time();
    format = g_strdup_printf("%s | %s", now, fmt);
    msg = g_strdup_vprintf(format, args);
    msg = check_for_end_of_line(msg);
    __print_msg( msg, 2 );

    va_end (args);
    g_free(now);
    g_free(msg);
    g_free(format);
}


static GtkTextView *_text_view = 0;
static GtkAdjustment *_adjust = 0;
static GtkTextTag *_error_tag = 0;
static GtkTextTag *_info_tag=0;
static GtkTextTag *_debug_tag=0;



static AlibavaMutex *get_the_mutex()
{
    static AlibavaMutex *_the_mutex=0;
    if (!_the_mutex)
    {
#if !GLIB_CHECK_VERSION(2,36,0)
        if (!g_thread_supported ())
            g_thread_init (NULL);
#endif
        _the_mutex = new AlibavaMutex;
    }
    return _the_mutex;
}

class MyTextBuffer
{
    private:
        GtkTextBuffer *buffer;
    public:
        MyTextBuffer();
        ~MyTextBuffer();

        GtkTextBuffer *the_buffer() { return buffer; }
};
MyTextBuffer::MyTextBuffer()
{
#if !GLIB_CHECK_VERSION(2,36,0)
    g_type_init();
#endif

    GtkTextTagTable *table = gtk_text_tag_table_new();
    buffer = gtk_text_buffer_new(table);

    _error_tag = gtk_text_buffer_create_tag(buffer,"error_tag",
                                            "foreground","red",NULL);
    _info_tag = gtk_text_buffer_create_tag(buffer,"info_tag",
                                           "foreground","black",NULL);
    _debug_tag = gtk_text_buffer_create_tag(buffer,"debug_tag",
                                            "foreground","lightsteelblue",NULL);

    g_object_ref(G_OBJECT(buffer));
}
MyTextBuffer::~MyTextBuffer()
{
    g_object_unref(G_OBJECT(buffer));
}
static MyTextBuffer _my_text_buffer_;


void set_text_buffer(char *ptr, char *adj)
{
    _text_view = GTK_TEXT_VIEW(ptr);
    gtk_text_view_set_buffer(GTK_TEXT_VIEW(ptr), _my_text_buffer_.the_buffer());
    _adjust = GTK_ADJUSTMENT(adj);
}

struct LineInfo
{
        gchar *msg;
        GtkTextTag *tag;
};

gboolean scroll_to_end(gpointer data)
{
    GtkTextBuffer *_text_buffer = _my_text_buffer_.the_buffer();
    GtkTextIter _iter;

    if (!_text_view)
        return FALSE;

    gtk_text_buffer_get_end_iter(_text_buffer, &_iter);
    gtk_text_view_scroll_to_iter(_text_view,&_iter,0.,FALSE,0.,0.);
    gtk_widget_queue_draw(GTK_WIDGET(_text_view));

    return FALSE;
}

gboolean write_text_to_buffer(gpointer data)
{
    LineInfo *txt = (LineInfo *)data;
    GtkTextBuffer *_text_buffer = _my_text_buffer_.the_buffer();
    GtkTextIter _iter;


    int nlines = gtk_text_buffer_get_line_count(_text_buffer);
    if (nlines>=1000)
    {
        GtkTextIter _start, _end;
        gtk_text_buffer_get_iter_at_line(_text_buffer, &_start,0);
        gtk_text_buffer_get_iter_at_line(_text_buffer, &_end,750);
        gtk_text_buffer_delete(_text_buffer,&_start, &_end);
    }
    gtk_text_buffer_get_end_iter(_text_buffer, &_iter);
    gtk_text_buffer_insert_with_tags(_text_buffer, &_iter, txt->msg, strlen(txt->msg), txt->tag,NULL);


    g_free(txt->msg);
    g_free(data);
    g_idle_add((GSourceFunc)scroll_to_end,NULL);

    return FALSE;
}

void __print_msg(const char *msg, int is_error)
{
    gchar *gmsg;

    get_the_mutex()->lock();

    GtkTextBuffer *_text_buffer = _my_text_buffer_.the_buffer();
    if (_text_buffer && __is_gui_session__ )
    {
        /**
         * Check the number of lines
         */
        GtkTextTag *_tag = NULL;
        switch (is_error)
        {
            case 0:
                _tag = _debug_tag;
                break;
            case 1:
                _tag = _error_tag;
                break;
            default:
                _tag = _info_tag;
                break;
        }
        LineInfo *data = g_new0(LineInfo, 1);
        data->msg = g_strdup(msg);
        data->tag = _tag;
        g_idle_add((GSourceFunc)write_text_to_buffer,data);
    }
    gmsg = g_strdup(msg);
//    gtmp = strrchr(gmsg,'\n');
//    if (gtmp)
//        *gtmp=0;

    switch (is_error)
    {
        case 0:
            g_print(gmsg, NULL);
            break;
        case 1:
            g_print(gmsg, NULL);
            break;
        case 2:
            g_print(gmsg, NULL);
            break;
        case 3:
            g_print(gmsg, NULL);
            break;
        default:
            g_print(gmsg, NULL);
            break;

    }
    g_free(gmsg);
    get_the_mutex()->unlock();
}
#endif

char *get_time()
{
    char outstr[256];
    time_t now;
    struct tm *tmp;

    time(&now);
    tmp = localtime(&now);

    strftime(outstr, 256, "%x - %X", tmp);
    char *gtim = g_strdup(outstr);
    char *ptr = gtim;
    for ( ;*ptr;ptr++)
    {
        if (*ptr=='\n')
        {
            *ptr=0;
            break;
        }
    }
    return gtim;
 }

AlibavaTimer __global_timer__;
void p_trace_msg(const gchar *func_name, int func_line, bool in, const gchar *fmt,...)
{
    gchar *format;
    gchar *msg;
    const gchar *prefix = ( in ? "<<<<< " : ">>>>> ");
    va_list args;

    va_start (args, fmt);
    format = g_strdup_printf("%s[%.6f %s:%d] %s", prefix, __global_timer__(), func_name, func_line, fmt);

    msg = g_strdup_vprintf(format, args);
    msg = check_for_end_of_line(msg);
    printf(msg);

    va_end (args);
    g_free(format);
    g_free( msg);
}


